##############################################################
##                 SAGE QASS                                  
##         Robust and Resistant Regression                    
##                                                            
##                Robert Andersen                             
##                12 December 2006                            
##############################################################

 
##Public opinion data for all countries (n=48)
Weakliem<-read.table('c:/Research/RobustRegression/Data/Weakliem.txt', header=T) 
Weakliem$GDP<-Weakliem$gdp/1000
Weakliem$SECPAY<-Weakliem$secpay-1
##Public opinion data for new democracies only (n=26)
Weakliem2<-read.table('c:/Research/RobustRegression/Data/Weakliem2.txt', header=T) 
Weakliem2$GDP<-Weakliem2$gdp/1000
Weakliem2$SECPAY<-Weakliem2$secpay-1

summary(Weakliem)
summary(Weakliem2)
 
### ###Creating simulated data with outliers to be used in sections 3 and 5.
    #x<-rnorm(30,10)
    x<- c(9.950292,  9.875331,  8.715756,  9.902030,  8.832989, 10.727488, 10.543382,
        10.879527,  9.330497, 10.536186,  9.944987, 10.334257, 10.064328,  9.751366,
        10.879872, 12.110906, 10.877009, 11.735530,  8.315679,  9.424651, 11.414522,
        9.503953,  8.573058,  8.172351, 10.539996,  9.152575, 11.060002,  9.709397,
        9.925195,  8.940158)

    #y<-5*x+rnorm(30,.2)
    y<-c(50.14906, 49.45526, 43.65610, 49.75638, 44.86117, 54.75021, 51.29881, 54.66010,
        48.26609, 54.87815, 50.17773, 53.94085, 51.18674, 52.12119, 55.20052, 61.23324,
        53.47144, 58.99026, 43.66051, 45.90653, 57.14173, 51.19621, 42.72159, 41.14141,
        54.02313, 46.88290, 54.88093, 49.19788, 48.55680, 45.24864)

    ###Good leverage point (x only)
    x1<-c(x, 15)
    y1<-c(y, ( -1.170+5.103*15)) 
    ###Bad leverage point--Influential observation (x and y given x)
    x2<-c(x, 7)
    y2<-c(y, 80)
    ###Bad outlier (y)
    x3<-c(x, mean(x))
    y3<-c(y, 5)
    #well-behaved data
    y<-c(y, 45.24864)
    x<-c(x, 8.940158)
   simdata<-data.frame(cbind(y,x,y1,x1,y2, x2,y3, x3)) 

##############################################
####Example 1: Weakliem et al--Nondemocracies
#######################
#WORLD VALUES SURVEY
#Nondemocracies only
########################




######################################################
##         CHAPTER 1                                  
######################################################

    ##ONLY one Table, and no figures. 
    ##Table for Marital Coital Frequency example


######################################################
##         CHAPTER 2                                  
######################################################

###Table 2.1 describes various M-estimators

##Figure 2.1  M-estimation weights with default tuning constants
##c=4.685 (Biweight); c=1.345 (Huber)
    biweight <- function(e) {
        ifelse (abs(e) > 4.685, 0, (1-(e/4.685)^2)^2)
        }
    Huberweight <- function(e) {
        ifelse (abs(e) > 1.345, 1.345/abs(e), 1)
        }
        e<-seq(-10, 10, by=.01)
    par(mfrow=c(1,3))
    plot(e,e, type="n", main="Mean",
        ylab="Weight (y)", xlab="y", lwd=2, ylim=c(0,1))
        abline(h=1, lwd=2)
    plot(e, biweight(e), type="l", main="Biweight Estimator",
        ylab="Weight (y)", xlab="y", lwd=2, ylim=c(0,1))
    
    plot(e, Huberweight(e), type="l", main="Huber Estimator",
        ylab="Weight (y)", xlab="y", lwd=2, ylim=c(0,1))

##Figure 2.2 Measures of location and an Extreme outlier
 

###Table 2.2   Measures of location and scale (with and without an Extreme outlier)

 ##Sourcing some functions for Robust Estimates from:
 #    Wilcox, Rand R. 2005. Introduction to Robust Estimation and Hypothesis Testing, 
 #            2nd Edition. London: Elsevier.
    source("http://psychology.usc.edu/rwilcox/Rallfunv1.v4")
    source("http://psychology.usc.edu/rwilcox/allfunv2.v4")


    ##simulated data
    par(mfrow=c(3,1))
    x<-rnorm(20,0,1)
    x2<-scale(x)
    x3<-c(x2, 60)
    #Location

    mean(x2);mean(x3)
    median(x2); median(x3)
    tmean(x2); tmean(x3) #Wilcox's code
    mest(x2); mest(x3)  #Wilcox's code


    #Scale
    sd(x2); sd(x3);sd(x3)/sd(x2)
        MDmean<-function(x){mean(abs(x-mean(x)))}
    MDmean(x2); MDmean(x3); MDmean(x3)/MDmean(x2)
        MDM<-function(x){median(abs(x-mean(x)))}
    MDM(x2); MDM(x3); MDM(x3)/MDM(x2)
    IQR(x2); IQR(x3); IQR(x3)/IQR(x2)
        MAD<-function(x){median(abs(x-median(x)))}
    MAD(x2); MAD(x3);MAD(x3)/MAD(x2)
    bivar(x2);bivar(x3); bivar(x3)/bivar(x2)  #Wilcox's code  


##Figure 2.2-Density estimate of public opinion variable
    plot(density(Weakliem2$SECPAY), xlab="Public Opinion towards Income Inequalty", main="")

 
###Table 2.4 Measures of location and scale for SECPAY, new democracies
    #location
    mean(Weakliem2$SECPAY);mean(Weakliem3$SECPAY)
    tmean(Weakliem2$SECPAY);tmean(Weakliem3$SECPAY)
    median(Weakliem2$SECPAY);median(Weakliem3$SECPAY)
    mest(Weakliem2$SECPAY);mest(Weakliem3$SECPAY)    
    #scale
    sd(Weakliem2$SECPAY);sd(Weakliem3$SECPAY); sd(Weakliem2$SECPAY)/sd(Weakliem3$SECPAY)
    MDmean(Weakliem2$SECPAY);MDmean(Weakliem3$SECPAY); MDmean(Weakliem2$SECPAY)/MDmean(Weakliem3$SECPAY)
    MDM(Weakliem2$SECPAY);MDM(Weakliem3$SECPAY); MDM(Weakliem2$SECPAY)/MDM(Weakliem3$SECPAY)
    IQR(Weakliem2$SECPAY);IQR(Weakliem3$SECPAY); IQR(Weakliem2$SECPAY)/IQR(Weakliem3$SECPAY)
    MAD(Weakliem2$SECPAY);MAD(Weakliem3$SECPAY); MAD(Weakliem2$SECPAY)/MAD(Weakliem3$SECPAY)
    bivar(Weakliem2$SECPAY);bivar(Weakliem3$SECPAY); bivar(Weakliem2$SECPAY)/bivar(Weakliem3$SECPAY)


############################################
##       Section 3  OLS and Outliers
#############################################
  
#####Figure 3.1
###Outlier graphs, using simdata
    attach(simdata)
   par(mfrow=c(1,3))
   plot(x3, y3, ylim=c(5,80), xlim=c(7,15), 
         xlab=expression(math(x[i])), ylab=expression(math(y[i])),
         main="A. Vertical outlier", pch=ifelse(y3<20, 19,1), cex=ifelse(y3<20, 2, 1.5))
    abline(lm(y3~x3), lty=2, lwd=2)
    abline(lm(y~x))
    text(locator(1), "A")
   plot(x1, y1, ylim=c(5,80), xlim=c(7,15), 
         xlab=expression(math(x[i])), ylab=expression(math(y[i])),
         main="B. 'Good' leverage point", pch=ifelse(y1>70, 19,1), cex=ifelse(y1>70, 2, 1.5))
    abline(lm(y1~x1), lty=2, lwd=2)
    abline(lm(y~x))
    text(locator(1), "B")
   plot(x2, y2, ylim=c(5,80), xlim=c(7,15), 
         xlab=expression(math(x[i])), ylab=expression(math(y[i])),
         main="C. 'Bad' leverage point", pch=ifelse(y2>70, 19,1), cex=ifelse(y2>70, 2, 1.5))
    abline(lm(y2~x2), lty=2, lwd=2)
    abline(lm(y~xa))
    text(locator(1), "C")
    detach(simdata)
    

##Table 3.1

###Using only 26 new democracies
attach(Weakliem2)
    Weakliem.model1<-lm(SECPAY~gini, data=Weakliem2)
    Weakliem.model2<-update(Weakliem.model1, subset=-c(7,26))
    library(MASS)
    Weakliem.model3<-rlm(SECPAY~gini, data=Weakliem2)
 
##Table 3.2
    
    Weakliem.model4<-lm(SECPAY~gini+GDP, data=Weakliem2)
    Weakliem.model4b<-update(Weakliem.model4, subset=-c(7,26))


#Figure 3.2
    par(mfrow=c(1,1))
    plot(gini, SECPAY,  xlab='Gini Coefficient', 
    ylab='Public Opinon regarding pay inequality', cex=1.5)
    identify(gini,SECPAY, row.names(Weakliem2))
    #"identify" returns cases 7, 26 as outliers
    abline(Weakliem.model1, lwd=2, lty=1, col=1) 
    abline(Weakliem.model2, lwd=2, lty=2, col=1)
    abline(Weakliem.model3, lwd=2, lty=3, col=1)
    legend(locator(1), lty=1:3, lwd=2, legend=c('OLS-All cases', 'OLS-Outliers excluded', 'Robust Regression'), bty="n")


##Figure 3.3
##Hat values

    #index plot of hat values allows us
    # to assess leverage
    plot(hatvalues(Weakliem.model4), 
        main="Hat Values for Model 1", cex=2)
    abline(h=c(2,3)*3/length(SECPAY), lty=2)
    #"h" signifies horizontal line
    #the average hat value=(k+1)/n.
    #A rule of thumb is that 2*average hat value
    #for large samples, and 3*average hat value
    #for small samples should be examined
    identify(1:length(SECPAY),
    hatvalues(Weakliem.model4), 
    row.names(Weakliem2))
    #Brazil has the largest hat-value--
    #much larger than the Czech Republic or Slovakia

###Figure 3.4
###Studentized Residuals
    library(car)  #car package by John Fox provides excellent diagnostics
    #qqplots allow us to detect outliers
     qq.plot(Weakliem.model4, simulate=T, labels=row.names(Weakliem2), cex=1.5, 
    ylab="Studentized Residuals for Model 1")   #from car package
    
    #several observations are outside
    #the 95% Confidence envelope
    outlier.test(Weakliem.model4)  #from car
    #returns a Bonferroni t-test for largest absolute 
    #studentized residual

###Figure 3.5
####Leverage
    ##DFBetas
    #dfbetas assess influence
    Weakliem.dfbetas<-dfbetas(Weakliem.model4)  #from car
   par(mfrow=c(1,2))
    plot(Weakliem.dfbetas[,2],  cex=1.5,
        ylab="DFBETA", main="Gini Coefficient")
    abline(h=2/sqrt(length(SECPAY)), lty=2)
    abline(h=-2/sqrt(length(SECPAY)), lty=2)
    abline(h=0, lty=1)
   identify(1:length(SECPAY), Weakliem.dfbetas[,2], 
        row.names(Weakliem2))
    plot(Weakliem.dfbetas[,3],  cex=1.5,
        ylab="DFBETA", main="per capita GDP")
        #c(2,3) specifies the coefficients of interest
    abline(h=2/sqrt(length(SECPAY)), lty=2)
    abline(h=-2/sqrt(length(SECPAY)), lty=2)
    abline(h=0, lty=1)
    identify(1:length(SECPAY),  Weakliem.dfbetas[,3], 
        row.names(Weakliem2))

###Figure 3.6
#Cook's D assesses influence
    par(mfrow=c(1,2))
    plot(cookd(Weakliem.model4), cex=1.5, main="Index plot of Cook's D", ylab="Cook's D") #from car
    abline(h=4/24, lty=2)
    identify(1:26, cookd(Weakliem.model4), row.names(Weakliem2))
    #Slovakia and Czech highest Cook's D,
    #But Slovenia also high
        
    #Bubble-plot displaying the studentized residuals,
    #hat-values, and Cook's D
    plot(hatvalues(Weakliem.model4),  
        rstudent(Weakliem.model4), ylim=c(-3,6),type='n',
        main="Influence Plot",
        xlab="Hat Values",
        ylab="Studentized Residuals")
    cook<-sqrt(cookd(Weakliem.model4))
    points(hatvalues(Weakliem.model4), 
        rstudent(Weakliem.model4), cex=10*cook/max(cook))
    abline(v=3/25, lty=2)#line for hatvalues
    abline(h=c(-2,0,2), lty=2)
    #lines for studentized residuals 
    identify(hatvalues(Weakliem.model4),  
    rstudent(Weakliem.model4), row.names(Weakliem2))

####Figure 3.7
#Added variable plots best for assessing joint influence
    par(mfrow=c(1,2))
    av.plot(Weakliem.model4, gini, main="Gini coefficient", 
           ylab="Public Opinion|others", xlab="Gini|Others") #from car
    av.plot(Weakliem.model4, GDP, main="per capita GDP")
    #again cases Czech Rep and Slovakia stand out
    # type "0" to end the plot
detach(Weakliem2) 


###############################################
##   Chapter 4--Robust Regression Estimators
################################################
library(MASS) ##for M and MM estimation
library(quantreg) ###for LAV regression

##################################
#######Figure 4.1
#################################
    attach(simdata)
   par(mfrow=c(1,3))
   plot(x3, y3, ylim=c(5,80), xlim=c(7,15), 
         xlab=expression(math(x[i])), ylab=expression(math(y[i])),
         main="A. Vertical outlier", pch=ifelse(y3<20, 19,1), cex=ifelse(y3<20, 2, 1.5))
    abline(lm(y3~x3), lty=1)#OLS
    abline(1.300195,  4.915847, lty=2)###LAV from qreg(x3, y3, qval=.5) from quantreg
    abline(rlm(y3~x3), lty=3)#M-estimation
    abline(1.27926, 4.919588, lty=4)##GM-estimation from chreg(x3,y3) ##Coakley-Hettmansperger
    abline(0.2876212, 5.0166087, lty=5)##S-estimation from Wilcox's stsreg(x3,y3)  
    abline(rlm(y3~x3, method="MM"), lty=6, lwd=1)##MM-estimation from MASS
    text(locator(1), "A")
        legend(locator(1), lty=1:6, 
        legend=c('OLS', 'LAV' ,'M-estimate (Huber)','GM-estimate','S-estimate','MM-estimate'))
    plot(x1, y1, ylim=c(5,80), xlim=c(7,15), 
         xlab=expression(math(x[i])), ylab=expression(math(y[i])),
         main="B. 'Good' leverage point", pch=ifelse(y1>70, 19,1), cex=ifelse(y1>70, 2, 1.5))
     abline(lm(y1~x1), lty=1)#OLS
    abline(0.8028577   4.9714762, lty=2)###LAV from qreg(x1, y1, qval=.5) from quantreg
    abline(rlm(y1~x1), lty=3)#M-estimation
    abline(1.122547, 4.940539, lty=4)##GM-estimation from chreg(x1,y1) ##Coakley-Hettmansperger
    abline(2.298709, 4.915847, lty=5)##S-estimation from Wilcox's stsreg(x1,y1)  
    abline(rlm(y1~x1, method="MM"), lty=6, lwd=1)##MM-estimation from MASS
    text(locator(1), "B")
   plot(x2, y2, ylim=c(5,80), xlim=c(7,15), 
         xlab=expression(math(x[i])), ylab=expression(math(y[i])),
         main="C. 'Bad' leverage point", pch=ifelse(y2>70, 19,1), cex=ifelse(y2>70, 2, 1.5))
   abline(lm(y2~x2), lty=1)#OLS
    abline( 1.863639,    4.867835 , lty=2)###LAV from qreg(x2, y2, qval=.5) from quantreg
    abline(rlm(y2~x2), lty=3)#M-estimation
    abline(1.420323, 4.90972, lty=4)##GM-estimation from chreg(x1,y1) ##Coakley-Hettmansperger
    abline(1.445808, 5.011582, lty=5)##S-estimation from Wilcox's stsreg(x1,y1)  
    abline(rlm(y2~x2, method="MM"), lty=6, lwd=1)##MM-estimation from MASS
    text(locator(1), "C")
    detach(simdata)

###################################
##Figure 4.2
#################################
    par(mfrow=c(1,3))
    plot(density(residuals(lm(y3~x3)), bw=1.5), main="A. Vertical outlier")
    plot(density(residuals(lm(y1~x1)), bw=1.5), main="B. 'Good' leverage point")
    plot(density(residuals(lm(y2~x2)), bw=1.5), main="'Bad' leverage point")


####Various ways for fitting robust  regression

    #M-estimates
        library(MASS)
        rlm(y~x) 
    #Generalized M-Estimates 
        bmreg(x,y)#Huber and Schweppe weights; Wilcox's code
        chreg(x,y) ##Coakley-Hettmansperger; ; Wilcox's code
    #S-Estimates
        stsreg(x,y) ##Wilcox's code
    #MM-estimates
        library(MASS)
        rlm(y~x, method="MM") 
                ##OR
        library(roblm)
        roblm(y~x, control = roblm.control(compute.roboot=TRUE) )      
    ##Least Absolute Values Regression
        library(quantreg)
        qreg(x, y, qval=.5)

###########################
###Table 4.2               
###                        

Weakliem2$GDP<-(Weakliem2$gdp)/1000
detach(Weakliem2)
attach(Weakliem2)
    #OLS estimates
        OLSest<-lm(SECPAY~gini+GDP) 
    #M-estimates
        library(MASS)
       #Huber
        Mest<-rlm(SECPAY~gini+GDP) 
       #Bisquare 
        Mest2<-rlm(SECPAY~gini+GDP, psi=psi.bisquare) 
    #Generalized M-Estimates 
        GMest<-bmreg(cbind(gini+GDP),SECPAY)#Huber and Schweppe weights; Wilcox's code
        CHest<-chreg(cbind(gini+GDP),SECPAY) ##Coakley-Hettmansperger; ; Wilcox's code
        
    
    #S-Estimates
        Sest<-stsreg(cbind(gini+GDP),SECPAY) ##Wilcox's code
    #MM-estimates
        library(MASS)
        MMest<-rlm(SECPAY~gini+GDP, method="MM") 
     ##Least absolute values regression
        library(quantreg)
        LAVest<-rq(SECPAY~gini+GDP, .5)

#############################################
####Table 4.3 Weights for robust regressions
####table of weights
    cook<-cookd(lm(SECPAY~gini+GDP))
    hats<-hatvalues(lm(SECPAY~gini+GDP))
    student<-rstudent(lm(SECPAY~gini+GDP))
    
    Huber<-Mest$w
    Bisquare<-Mest2$w
    MM<-MMest$w
    GM<-CHest$w ##Coakley-Hettmansperger
 


####Figure 4.3--index plots of weights
    par(mfrow=c(1,3))
    plot(Mest$w, ylab="Weight", main="M-estimate", cex=1.5)
    identify(1:length(Mest$w),
    Mest$w, 
    row.names(Weakliem2))
    plot(GMest$w, ylab="Weight", main="GM-estimate", cex=1.5)
    identify(1:length(GMest$w),
    GMest$w, 
    row.names(Weakliem2))
    plot(MMest$w, ylab="Weight", main="MM-estimate", cex=1.5)
    identify(1:length(MMest$w),
    MMest$w, 
    row.names(Weakliem2))


######Figure 4.4
    #RR-plots

par(mar=(c(5, 4, 1, 1)))
par(mfrow=c(3,3))

plot(residuals(LAVest), residuals(OLSest), xlim=c(-.1,.6), ylim=c(-.1,.6),
       xlab="LAV", ylab="OLS") 
       abline(0,1, lty=2); abline(lm(residuals(OLSest)~residuals(LAVest)))
       identify(residuals(LAVest),residuals(OLSest), row.names(Weakliem2))
    plot(residuals(LAVest), residuals(OLS), type="n", bty="n", fg="white", col.axis="white", col.lab="white")
    plot(residuals(LAVest), residuals(OLS), type="n", bty="n", fg="white", col.axis="white", col.lab="white")

plot(residuals(GMest), residuals(OLSest), xlim=c(-.1,.6), ylim=c(-.1,.6), 
       xlab="GM-Estimate", ylab="OLS")
       abline(0,1, lty=2); abline(lm(residuals(OLSest)~residuals(GMest)))
        identify(residuals(GMest),residuals(OLSest), row.names(Weakliem2))
plot(residuals(GMest), residuals(LAVest), xlim=c(-.1,.6), ylim=c(-.1,.6), 
        xlab="GM-Estimate", ylab="LAV") 
        abline(0,1, lty=2); abline(lm(residuals(LAVest)~residuals(GMest)))
        identify(residuals(GMest),residuals(LAVest), row.names(Weakliem2))
plot(residuals(LAVest), residuals(OLSest), type="n", bty="n", fg="white", col.axis="white", col.lab="white")
 
plot(residuals(MMest), residuals(OLSest), xlim=c(-.1,.6), ylim=c(-.1,.6), 
        xlab="MM-Estimate", ylab="OLS") 
        abline(0,1, lty=2); abline(lm(residuals(OLSest)~residuals(MMest)))
        identify(residuals(MMest),residuals(OLSest), row.names(Weakliem2))
plot(residuals(MMest), residuals(LAVest), xlim=c(-.1,.6), ylim=c(-.1,.6), 
        xlab="MM-Estimate",ylab="LAV") 
        abline(0,1, lty=2); abline(lm(residuals(LAVest)~residuals(MMest)))
        identify(residuals(MMest),residuals(LAVest), row.names(Weakliem2))
plot(residuals(MMest), residuals(GMest), xlim=c(-.1,.6), ylim=c(-.1,.6), 
        xlab="MM-Estimate", ylab="GM-Estimate")
        abline(0,1, lty=2); abline(lm(residuals(GMest)~residuals(MMest)))
        identify(residuals(MMest),residuals(GMest), row.names(Weakliem2))




#######Figure 4.5
###Robust residual (LTS) versus Robust Distances
    library(robustbase)
    lts.weakliem2 <- ltsReg(SECPAY ~ gini+gdp, Weakliem2)
    plot(lts.weakliem2, which = "rdiag", main="") 



################################################
## Chapter 5--- Statistical Inference for 
##  Robust Regression, including Bootstrapping
################################################


############################################
#Table 5.1 #Estimates AND standard errors
#########################################

    ####USING all countries:
    detatch(Weakliem2)
    attach(Weakliem)
    ###M-estimation, Huber weight
        library(MASS)
        Weakliem.Huber<-rlm(SECPAY~democrat*gini+GDP)
    ###MM-Estimation
        Weakliem.MM<-rlm(SECPAY~democrat*gini+GDP, method='MM')
        ###Alternatively, 
        library(roblm)
        summary(roblm(SECPAY~democrat*gini+GDP, control = roblm.control(compute.roboot=TRUE) ) )
    ###OLS, all cases
        Weakliem.lm<-lm(SECPAY~democrat*gini+GDP)
        ###OLS, ouliers removed
        library(car)
        av.plots(Weakliem.lm)  ###finding and removing outliers
        Weakliem.lm2<-update(Weakliem.lm, subset=-c(24,48))
        library(MASS)

        summary(Weakliem.lm)
        summary(Weakliem.lm2)
        summary(Weakliem.Huber)
        summary(Weakliem.MM)

##Fixed-x Bootstrap for Huber
    library(boot)
    boot.huber.fixed<-function(data, indices, maxit=20){
        fit<-fitted(Weakliem.Huber)
        e<-residuals(Weakliem.Huber)
        X<-model.matrix(Weakliem.Huber)  
        y<-fit+e[indices]
        mod<-rlm(y~X-1, maxit=maxit)
        #y~X-1 omits the intercept--line goes through the origin
        coefficients(mod)
        }

    Weakliem.huber.fixed<-boot(Weakliem, boot.huber.fixed, R=1000, maxit=100)
    Weakliem.huber.fixed
    #diagnostic plot
    par(mfrow=c(5,1))
    Weakliem.Huber

##########################################################   
###Figure 5.1--plot of boot for Democracy coefficeint only
###Others are given below as well, however
    plot(Weakliem.huber.fixed, index=2)
    plot(Weakliem.huber.fixed, index=2)
    plot(Weakliem.huber.fixed, index=3)
    plot(Weakliem.huber.fixed, index=4)
    plot(Weakliem.huber.fixed, index=5)
    jack.after.boot(Weakliem.huber.fixed, index=2)
    #Bootstap confidence intervals
    boot.ci(Weakliem.huber.fixed, index=1)
    boot.ci(Weakliem.huber.fixed, index=2)
    boot.ci(Weakliem.huber.fixed, index=3)

        ###Examples of random-x bootstrapping using Wilcox's code
        #95% bootstrapped CI 
        rregci(x, SECPAY, regfun=bmreg,nboot=100,alpha=.05)
        ##Coakley-Hettmansperger Estimator
        Weakliem.chreg<-chreg(cbind(gini, gdp), SECPAY)
        rregci(x, SECPAY, regfun=chreg,nboot=100,alpha=.05)
        ##Skipped Estimators
        mgvreg(cbind(gini,gdp),SECPAY)
        rregci(x, SECPAY, regfun=mgvreg,nboot=100,alpha=.05)
        ## Deepest regression line (only one predictor possible)
        Weakliem.depreg<-depreg(gini, SECPAY)


###################################
#          Table 5.2               
###################################

        summary(Weakliem.model1)
        summary(Weakliem.model2)
        summary(Weakliem.model3)


########################################
##           Figure 5.2 Effect display
########################################
    #####Outliers included
    par(mfrow=c(1,2))
    plot(gini,SECPAY, type="n", xlab="Gini coefficient", ylab="Public opinion",
        ylim=c(1.0, 1.35), main="OLS Regression")
    abline(lm((c(1.26,1.14))~(c(19.5,51.1))), lty=1, lwd=2)#new democracies
    abline(lm((c(1.19,1.18))~(c(19.5,51.1))), lty=2, lwd=2)#old democracies
    legend(locator(1), lwd=2,lty=c(2,1),legend=c('Old democracies','New democracies'), bty="n")  
   
     
    #####MM-estimation
    plot(gini,SECPAY, type="n", xlab="Gini coefficient", ylab="Public opinion",
        ylim=c(1.0, 1.35), main="MM-estimation")
    abline(lm((c(1.04,1.19))~(c(19.5,51))), lty=1, lwd=2)#new democracies
    abline(lm((c(1.31,1.12))~(c(19.5,51))), lty=2, lwd=2)#old democracies
    legend(locator(1),lty=c(2,1), lwd=2,legend=c('Old democracies','New democracies'), bty="n")  
   



##############################
## Chapter 6 Robust GLMs      
##############################

################################################################
#  Logistic regression  
#  Data from British Election Panel Study, 
#  used in 
#     Andersen, R. and G. Evans (2003) 'Who Blairs Wins? Leadership and voting in the 2001 election', 
#     British Elections & Parties Review, 13: 229-247. 
#################################################################
    library(car)
    beps<-read.table('C:/Research/RobustRegression/Data/beps2001.txt', header=T)
    beps$novote<-recode(beps$VOT01, "'novote'='Novote'; else='vote'")
    beps$Labour<-recode(beps$VOT01, "'Labour'='Labour'; else='other'")
    beps$Labour97<-recode(beps$VOT97, "'Labour'='Labour'; else='other'")
    beps$novote<-relevel(beps$novote, 'vote')
    beps$EDUCAT<-relevel(beps$EDUCAT, 'none')
    beps$CLASS<-relevel(beps$CLASS, 'Workclas')
    beps$Labour<-relevel(beps$Labour, 'other')
    
    newData<-na.omit(subset(beps, select=c(AGE,GENDER,CLASS,EDUCAT, SOCIPAST,EGOPAST,BLAIR,Labour, Labour97)))
    
    mod1 <- glm(Labour ~ AGE+GENDER+EDUCAT+CLASS+SOCIPAST+EGOPAST+BLAIR, family=binomial, data=newData)
        
    ###Figure 6.2###########
    ########################    
    plot(cookd(mod1), ylab="Cook's D",main="Index plot of Cook's D", cex=1.5)
            identify(1:length(newData$Labour), cookd(mod1), row.names(newData))
 
    #Robust logistic regresssion    
    library(robustbase)
    mod2 <- glmrob(Labour ~ AGE+GENDER+EDUCAT+CLASS+SOCIPAST+EGOPAST+BLAIR, family=binomial, data=newData)
    ########################
    #####Table 6.2
    ################
    summary(mod1); summary(mod2)
    exp(coef(mod1)); exp(coef(mod2)) 


#######################################################
#Poisson regression model example                             
#######################################################
#Using Canadian data from Equality, Security, and Community (ESC) survey of 2000,
#concentrating only on Quebec

    library(foreign) #foreign library used to bring an SPSS data file
    cesc<-read.spss("C:/Research/RobustRegression/Data/cesc2000.sav", 
        use.value.labels=TRUE, to.data.frame=TRUE)
    cesc$Region2<-recode(cesc$REGION,"'Quebec'='Quebec'; 'Ontario'='Ontario'; 'Atlantic'='Atlantic'; else='West'")
    Quebec<-na.omit(subset(cesc, Region2=="Quebec", drop=TRUE))

    ###Poisson model
    mod1<-glm(TOTORG~MEN+CANBORN+LANG, 
                data=Quebec, family=poisson)
                
    
    #Diagnostics
    
    ##########################################################
    ####Figure 6.3   Diagnostic plots for the Poisson model
    ##########################################################
    par(mfrow=c(1,2))
    plot(cookd(mod1), main="Index plot of Cook's D", ylab="Cook's D")
    identify(1:length(Quebec$TOTORG), cookd(mod1), row.names(Quebec))
    
    plot(dfbetas(mod1)[,5], main="DFBETAs for 'Canadian born' coefficient",
       ylab="DFBETAs")
    identify(1:length(Quebec$TOTORG), dfbetas(mod1)[,5], row.names(Quebec))
    
    
    ##Robust GLM Poisson regression
    mod2<-glmrob(TOTORG~MEN+CANBORN+LANG, 
                data=Quebec, family=poisson)

              
##########################################################
##               Table 6.2                                
##       Summaries Poisson and Robust Poisson regressions 
##########################################################

    ##Robust GLM Poisson regression
    library(robustbase)
    mod2<-glmrob(TOTORG~MEN+CANBORN+LANG, 
                data=Quebec, family=poisson)
    
    summary(mod1)
    exp(mod1$coefficients)
    
    summary(mod2)
    exp(mod2$coefficients)

    
